 /******************************************************************************
 * Scanner for Verilog 2001 subset
 * written by M. Kreis
 * Date: 04/2008
 * supports the IEEE Std 1364-2001 (Revision of IEEE Std 1364-1995)Verilog subset
 *
 *****************************************************************************/


%{

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <ctype.h>
#include "commentscan.h"
#include "verilogscanner.h"
#include "doxygen.h"
#include "searchindex.h"
#include "VerilogDocGen.h"
#include <ctype.h>
#include "scanner.h"
#include "vhdldocgen.h"
//--------------------------------------------------------------------------

static ParserInterface *g_thisParser;

static int yyLineNr = 1;
static int yyPrevLine=1;
static int yyEndLine=1;
static char* g_buf;

static bool gInit=false;
static int inputPosition;
static Entry* curRoots;

static int startComment;
static int count,total;
static int iSize=0;
static int currToken;
static const char* inString;
static QFile inputFile;
static QCString yyFileName;
static QList<QCString> qlist;
static QList<Entry> qfeatlist;
static MyParserConv *pconv;
static QCString lastLetter;
static int totLines=0;


// ----------------- <verilog-code > ----------------------------------

static QCString   g_CurrClass;
static QCString   g_tempComp;
static CodeOutputInterface * g_code;

static ClassSDict    *g_codeClassSDict = 0;
static QCString      g_curClassName;
static const char *  g_inputString;     //!< the code fragment as text
static int           g_inputLines=0;      //!< number of line in the code fragment
static bool          g_needsTermination;

static QCString      g_exampleName;
static QCString      g_exampleFile;

static QCString      g_CurrScope;
static FileDef *     g_sourceFileDef;
static Definition *  g_currentDefinition;
static MemberDef *   g_currentMemberDef;
static bool          g_includeCodeFragment;
static const char *  g_currentFontClass;
static bool          g_lexInit = FALSE;
static bool          g_parseCode=FALSE;

// ------------------< functions for verilog scanner >---------------

static void writeFont(const char *s,const char* text);
static void generateMemLink(CodeOutputInterface &ol,QCString &clName,QCString& memberName);
static bool writeColoredWord(QCString& word );
static void startCodeLine();
static bool generateClassOrGlobalLink(CodeOutputInterface &ol,char *clName, bool typeOnly=FALSE);
static bool generateMemLink(CodeOutputInterface &ol,QCString &clName,QCString& memberName,int type);
static void codifyLines(char *text,char *cl=NULL);
static void addToSearchIndex(const char *text);
static void writeMultiLineCodeLink(CodeOutputInterface &ol,
                  const char *ref,const char *file,
                  const char *anchor,const char *text);

static void startFontClass(const char *s);
static void endFontClass();
static void parseFeatures();

//-------------------------------------------------------------------

#define YY_NEVER_INTERACTIVE 1

//-----------------------------------------------------------

static QDict<QCString> verilogKeyDict(17,false);
static QDict<QCString> verilogFeatureDict(17,false);
static QDict<QCString> verilogDefineDict(17,false);

static const char* VerilogKewWordMap[] = {"module","endmodule","input","output","inout","reg","parameter","always",  
	"case","endcase","if","begin","posedge","negedge","or","wire","tri","tri0","instance","use","design","config","endconfig","include","library","liblist",
	"function","endfunction","assign","end","for","else","initial","integer","real","task",
	"and","buf","bufif0","bufif1","casex","casez","cmos","deassign","default","defparam","disable",
	"edge","endattribute","endprimitive","endspecify","endtable","endtask","event",
    "force","forever","fork","force","forever","ifnone","join","large","macromodule","medium","Input","Output","Inout",
	"nand","nmos","nor","not","notif0","notif1","localparam",
	"pmos","primitive","pull0","pull1","pullup","pulldown","automatic",
	"rcmos","real","time","realtime","release","repeat","rnmos","rpmos","rtran","rtranif0","rtranif1",
	"scalared","signed","small","specify","specparam","strength","strong0","strong1","supply0","supply1",
	"table","tran","tranif0","tranif1","tri1","triand","trior","trireg","pulsestyle_ondetect",    "showcancelled","noshowcancelled","pulsestyle_onevent",
	"generate","endgenerate","genvar","unsigend","xor","xnor","wait","wand","weak0","weak1","while","wire","wor","vectored",""}; 

static const char* VerilogDefineMap[] = {"`line","`resetall","`define","`undef","`timescale","`ifndef","`ifdef","`else","`endif","`include","`celldefine","`endcelldefine","`default_nettype","`unconnected_drive","`nounconnected_drive",""};

// some but not all system words
static const char* VerilogKewWordMap1[] = {"$async$and$array","$async$and$plane","$async$nand$array", "$async$nand$plane","$async$or$array", "$async$or$plane",
"$async$nor$array", "$async$nor$plane","$sync$and$array", "$sync$and$plane","$sync$nand$array", "$sync$nand$plane",
"$sync$or$array", "$sync$or$plane","$sync$nor$array", "$sync$nor$plane","$display", "$strobe",
"$displayb", "$strobeb","$displayh", "$strobeh","$displayo","$strobeo","$monitor", "$write","$monitorb", "$writeb",
"$monitorh", "$writeh","$monitoro", "$writeo","$monitoroff", "$monitoron","$fclose", "$fopen","$fdisplay", "$fstrobe",
"$fdisplayb", "$fstrobeb","$fdisplayh", "$fstrobeh","$fdisplayo", "$fstrobeo","$fgetc", "$ungetc","$fflush", "$ferror",
"$fgets", "$rewind","$fmonitor", "$fwrite","$fmonitorb", "$fwriteb","$fmonitorh",  "$fwriteh",
"$fmonitoro", "$fwriteo","$readmemb", "$readmemh","$swrite", "$swriteb","$swriteo", "$swriteh",
"$sformat", "$sdf_annotate","$fscanf", "$sscanf","$fread", "$ftell","$fseek", "$printtimescale", 
"$timeformat" ,"$finish", "$stop", "$realtime","$stime", "$time","$bitstoreal", "$realtobits",
"$itor", "$rtoi","$signed", "$unsigned","$test$plusargs", "$value$plusargs","$q_initialize", "$q_add",
"$q_remove", "$q_full","$q_exam",""};


static void buildKeyMap()
{
	int j=0;
    verilogFeatureDict.setAutoDelete(true);
    verilogKeyDict.setAutoDelete(true);
    verilogDefineDict.setAutoDelete(true);
	qlist.setAutoDelete(true);
	qfeatlist.setAutoDelete(true);
 
	QCString p=VerilogKewWordMap[0];
    
	while(!p.isEmpty())
	{
		verilogKeyDict.insert(p,new QCString(p.data()));
    	p=VerilogKewWordMap[++j];	  
	}

	j=0;
    p=VerilogKewWordMap1[0]; 
	while(!p.isEmpty())
	{
		verilogFeatureDict.insert(p,new QCString(p.data()));
    	p=VerilogKewWordMap1[++j];	  
	}

   j=0;
    p=VerilogDefineMap[0]; 
	while(!p.isEmpty())
	{
		verilogDefineDict.insert(p,new QCString(p.data()));
    	p=VerilogDefineMap[++j];	  
	}

}// buildKeypMap

int VerilogDocGen::findKeyWord(const char *str)
{
    QCString word(str);
	if (word.isEmpty() || word.at(0)=='\0') return 0;
	
	if(verilogKeyDict.find(word))
	  return 1;
 
    if(verilogFeatureDict.find(word))
	  return 2;
   
   return 0;
}

void writeVerilogFont(const char* col,const char* text)
{
  writeFont(col,text);
}

// writes the a word with its  color to the output

static void writeVWord(QCString& qcs){
   if(qcs.isEmpty())return;
    QCString temp=qcs;
	if(qcs.stripPrefix("\n")){
	     codifyLines(temp.data());return;}
	else if(qcs.stripPrefix(" "))
	     {g_code->codify(temp.data()); return;}   
 	if(qcs.stripPrefix("\""))
        {writeFont("keyword",temp.data());return;}
	if(qcs.stripPrefix("/*")){
	     startFontClass("keyword");
	     codifyLines(temp.data());
		 endFontClass();
		 return;					
		}
     if(qcs.stripPrefix("//"))
        {writeFont("keyword",temp.data());return;}
	 if (VerilogDocGen::findKeyWord(qcs.data())==1)
         {writeFont("vhdlkeyword",qcs.data());return;}
   	 if(qcs.stripPrefix("`"))
	     { writeFont("stringliteral",temp.data());
		//  printf("\n [[[%s]]\n",temp.data());
		  return;
		  }
     if(qcs.length()==1)
         {codifyLines(qcs.data());return;}
     if(qcs.stripPrefix("$"))
          writeFont("vhdlkeyword",temp.data());
     else g_code->codify(temp.data());
	  
}//writeVWord


// writes the rest of the input buffer
void printVerilogBuffer(bool b)
{
  QCString qbuffer;
  uint len=qlist.count();
  for(uint j=0;j<len;j++) {
    QCString *qcs=(QCString*)qlist.at(j);
	writeVWord(*qcs);
        
 }//for
 qlist.clear();
}//printVerilogBuffer


void writePrevVerilogWords(QCString& qcs){

 uint len=qlist.count();
 uint v=0;
  
  for(uint j=0;j<len;j++) {
    QCString *word=(QCString*)qlist.at(j);
	 
	 if(strcmp(qcs.data(),word->data()) != 0) 
      writeVWord(*word);
     else { 
	 v=j; 
	 break; 
	 }
   }//for
	 
   if(v>=(len-1))
    { 
	 qlist.clear();
	 return; 
	 }
    v++;
    while(v>0){ qlist.removeFirst(); v--;}
} // writePrevVerilogWords


void printVerilogStringList()
{
  QCString qbuffer;
  int len=qlist.count();
  fprintf(stderr,"\n----------------------------------\n");
  for(int j=0;j<len;j++) {
    QCString *qcs=(QCString*)qlist.at(j);
	 qbuffer+=qcs->data();
  }
  fprintf(stderr,"\n[%s]",qbuffer.data());
  fprintf(stderr,"\n----------------------------------\n");
}


void codifyVerilogString(char* c,char* color)
{
 if(color==NULL)
  g_code->codify(c);
 else
  writeFont(color,c);
 }

bool  generateVerilogMemLink(QCString &clName,QCString& memberName,int type)
{
 return generateMemLink(*g_code,clName,memberName,type);
}


static bool generateMemLink(CodeOutputInterface &ol,QCString &clName,QCString& memberName,int type)
{
  if(clName.isEmpty() || memberName.isEmpty()) return false; 

  bool isLocal=false;

  MemberDef *md=0;
  if(memberName.contains('`'))
   isLocal=true;


   md=VerilogDocGen::findMember(clName,memberName,type);
  
   if(md==0)
   md=VerilogDocGen::findMember(clName,memberName,-1);
  
 
  if (md && md->isLinkable()) // is it a linkable class
  {
    addToSearchIndex(memberName);
    if(isLocal) memberName.prepend("`");
	 writeMultiLineCodeLink(ol,md->getReference(),md->getOutputFileBase(),md->anchor(),memberName);
   return true;
  }

 // nothing found, just write out the word
 //   ol.linkableSymbol(yyLineNr,memberName,0,g_currentMemberDef?g_currentMemberDef:g_currentDefinition);
 //   codifyLines(memberName.data());
    addToSearchIndex(memberName);

 return false;
}// generateMemLink



static void endCodeLine()
{
  if (g_currentFontClass) { g_code->endFontClass(); }
  g_code->endCodeLine();
}

static void addToSearchIndex(const char *text)
{
  static bool searchEngineEnabled=Config_getBool("SEARCHENGINE");
  if (searchEngineEnabled)
  {
    Doxygen::searchIndex->addWord(text,FALSE);
  }
}


/*! writes a link to a fragment \a text that may span multiple lines, inserting
 * line numbers for each line. If \a text contains newlines, the link will be 
 * split into multiple links with the same destination, one for each line.
 */
static void writeMultiLineCodeLink(CodeOutputInterface &ol,
                  const char *ref,const char *file,
                  const char *anchor,const char *text)
{
  bool done=FALSE;
  char *p=(char *)text;
  while (!done)
  {
    char *sp=p;
    char c;
    while ((c=*p++) && c!='\n');
    if (c=='\n')
    {
      yyLineNr++;
      *(p-1)='\0';
     // printf("writeCodeLink(%s,%s,%s,%s)\n",ref,file,anchor,sp);
      ol.writeCodeLink(ref,file,anchor,sp,0);
      endCodeLine();
      if (yyLineNr<g_inputLines) 
      {
	startCodeLine();
      }
    }
    else
    {
    //  printf("writeCodeLink(%s,%s,%s,%s)\n",ref,file,anchor,sp);
      ol.writeCodeLink(ref,file,anchor,sp,0);
      done=TRUE;
    }
  }
}

bool generateVerilogClassOrGlobalLink(char *clName)
{
  return generateClassOrGlobalLink(*g_code,clName,FALSE);
}


static bool generateClassOrGlobalLink(CodeOutputInterface &ol,char *clName, bool typeOnly)
{
  QCString className=QCString(clName);

  if(className.isEmpty()) return false;

  ClassDef *cd=0,*lcd=0;
  MemberDef *md=0;
  bool isLocal=FALSE;
  cd = getClass(className.data()); 
  
  if (cd && cd->isLinkable()) // is it a linkable class
  {
  	  QCString temp=cd->displayName();
	  ol.linkableSymbol(yyLineNr,temp,cd, g_currentMemberDef ? g_currentMemberDef : g_currentDefinition);
      writeMultiLineCodeLink(ol,cd->getReference(),cd->getOutputFileBase(),0,temp);
      addToSearchIndex(className);
   return true;
  }
 return false;
}// generateClassOrGLink  


static void startFontClass(const char *s)
{
  if(s==NULL) return;
  g_code->startFontClass(s);
  g_currentFontClass=s;
}

static void writeFont(const char *s,const char* text)
{
  if(s==NULL) return;
  g_code->startFontClass(s);
  g_code->codify(text);
  g_code->endFontClass();
}



/*! counts the number of lines in the input */
static int countLines()
{
  const char *p=inString;
  char c;
  int count=1;
  while ((c=*p)) 
  { 
    p++ ; 
    if (c=='\n') count++;  
  }
  if (p>g_inputString && *(p-1)!='\n') 
  { // last line does not end with a \n, so we add an extra
    // line and explicitly terminate the line after parsing.
    count++, 
    g_needsTermination=TRUE; 
  } 
  return count;
}

/*! writes a word to the output.
 *  If curr_class is defined, the word belongs to a class
 *  and will be linked.
 */

static void writeWord(char *word,char* curr_class=NULL)
{
  
  if(word==NULL)return;
   if(VerilogDocGen::findKeyWord(word))
     writeFont("vhdlkeyword",word);
   else
   g_code->codify(word); 
  return;
}// writeWord



/*! write a code fragment `text' that may span multiple lines, inserting
 * line numbers for each line.
 */
static void codifyLines(char *text,char *cl)
{
 // printf("codifyLines(%d,\"%d\")\n",yyLineNr,strlen(text));
  if(text==NULL) return;
  char *p=text,*sp=p;
  char c;
  bool done=FALSE;
  while (!done)
  {
    sp=p;
    while ((c=*p++) && c!='\n');
    if (c=='\n')
    {
      yyLineNr++;
      *(p-1)='\0';
       writeWord(sp,cl);
	  endCodeLine();
      if (yyLineNr<g_inputLines) 
      {
    	startCodeLine();
      }
    }
    else
    {
      writeWord(sp,cl);
	  done=TRUE;
    }
  }
}


static void setCurrentDoc(const QCString &name,const QCString &base,const QCString &anchor="")
{
  static bool searchEngineEnabled=Config_getBool("SEARCHENGINE");
  if (searchEngineEnabled)
  {
    Doxygen::searchIndex->setCurrentDoc(name,base,anchor);
  }
}


/*! start a new line of code, inserting a line number if g_sourceFileDef
 * is TRUE. If a definition starts at the current line, then the line
 * number is linked to the documentation of that definition.
 */
static void startCodeLine()
{
  //if (g_currentFontClass) { g_code->endFontClass(); }
  if (g_sourceFileDef)
  {
    // if((yyLineNr % 500) == 0) 
	// fprintf(stderr,"\n starting Line %d:",yyLineNr);
    Definition *d   = g_sourceFileDef->getSourceDefinition(yyLineNr);
   // printf("startCodeLine %d d=%s\n", yyLineNr,d ? d->name().data() : "<null>");
    if (!g_includeCodeFragment && d)
    {
      g_currentDefinition = d;
      g_currentMemberDef = g_sourceFileDef->getSourceMember(yyLineNr);
      if(!g_tempComp.isEmpty() && g_currentMemberDef )
	  {
        ClassDef *cf=VhdlDocGen::getClass(g_tempComp.data());
	    QCString nn=g_currentMemberDef->name();
		MemberDef* mdeff=VhdlDocGen::findMember(g_tempComp,nn);
		if(mdeff)
          g_currentMemberDef=mdeff;
	  
	  }
       
       QCString lineAnchor;
      lineAnchor.sprintf("l%05d",yyLineNr);
      if (g_currentMemberDef)
      {
     	g_code->writeLineNumber(g_currentMemberDef->getReference(),
	                        g_currentMemberDef->getOutputFileBase(),
	                        g_currentMemberDef->anchor(),yyLineNr);
        setCurrentDoc(
                                g_currentMemberDef->qualifiedName(),
	                        g_sourceFileDef->getSourceFileBase(),
	                        lineAnchor);
      }
      else if (d->isLinkableInProject())
      {
        g_code->writeLineNumber(d->getReference(),
	                        d->getOutputFileBase(),
	                        0,yyLineNr);
        setCurrentDoc(
                                d->qualifiedName(),
	                        g_sourceFileDef->getSourceFileBase(),
	                        lineAnchor);
      }
    }
    else
    {
      g_code->writeLineNumber(0,0,0,yyLineNr);
    }
  }
  g_code->startCodeLine(); 
  if (g_currentFontClass)
  {
    g_code->startFontClass(g_currentFontClass);
  }
}


static void endFontClass()
{
  if (g_currentFontClass)
  {
    g_code->endFontClass();
    g_currentFontClass=0;
  }
}

//---------------------------------------------------------------

static void addText(char* c,int len);
static void addToken(char c);
static int startLex();
static void resetScanner(const char* s,MyParserConv* parse);


static void addOneTextLine(QCString& ss )
{

    if(g_parseCode) return;
	Entry*  pTemp=VhdlDocGen::getEntryAtLine(curRoots,yyLineNr) ;
							 
	 if(pTemp)
	 {
      ss=ss.stripWhiteSpace();
      ss.stripPrefix("--#");
	  pTemp->brief=ss;
	  pTemp->briefLine=yyLineNr;
	  return;
    }		

    if(qfeatlist.count()){
      pTemp=(Entry*)qfeatlist.last();
      if(yyLineNr!=pTemp->briefLine) return;
	  pTemp->brief=ss;
	  pTemp->briefLine=yyLineNr;
	}
}


#undef	YY_INPUT
#define	YY_INPUT(buf,result,max_size) result=verilogScanYYread(buf,max_size);


static int verilogScanYYread(char *buf,int max_size)
{
  
  int c=0;
    if(false)
    {
       c = inputFile.readBlock(buf,max_size);
       if (c==-1) yy_fatal_error("input in flex scanner failed");
    }
    else
    {
  
   while ( c < max_size && inString[inputPosition] )
  {
    *buf = inString[inputPosition++] ;
    c++; buf++;
  }
  }
  return c;
}


static bool check(bool bb=true)
{
  bool b=true;
  char c=yy_hold_char;
 

  if(iSize==0){
   if(g_parseCode){
      qlist.append(new QCString(verilogScanYYtext)); 
  }
  return true;
 }
 
 
  char d = g_buf[iSize-1];
  int l=isdigit(c);
  int k=isdigit(d);

  if(k!=0 || l!=0 || d=='$') return false;

  l=islower(c);
  k=islower(d);

  if(k!=0 || l!=0) 
    return false;

  l=isupper(c);
  k=isupper(d);

  if(k!=0 || l!=0) 
    return false;

  if(c=='_' || d=='_') return false;
  					               
  if(g_parseCode){
   qlist.append(new QCString(verilogScanYYtext));
  }

  return true;
}

static void parseToken(const char* s){
     if(g_parseCode)    
	 qlist.append(new QCString(verilogScanYYtext));
	 c_lval.ctype=s[0];
	 addToken(s[0]);
	}

%}

/* -------------- VERLIOG SECTION -----------------------------------*/

LF          [ \t\n]
COMMENT     "//"[^\n]*
COMMENT1    "//%"[^\n]*
MIT  "*/"
STRING      ["][^"]*["]
FEATURE     ("`include"|"`line"|"`timescale"|"`define"|"`ifndef"|"`ifdef"|"`else"|"`endif"|"`elsif"|"`undef")[^\n]+
FEATURE1    [`][a-zA-Z0-9_]+
SIGNEDBASE  ['][sS][oOhHbBdD]
BASE	    "'b"|"'B"|"'o"|"'O"|"'d"|"'D"|"'h"|"'H"|{SIGNEDBASE}
NETTYPE	    "wire"|"tri"|"tri1"|"supply0"|"wand"|"triand"|"tri0"|"supply1"|"wor"|"trior"|"trireg"
STRENGTH1	"supply1"|"strong1"|"pull1"|"weak1"|"highz1"
STRENGTH0	"supply0"|"strong0"|"pull0"|"weak0"|"highz0"
GATETYPE  	"and"|"nand"|"or"|"nor"|"xor"|"xnor"|"buf"|"bufif0"|"bufif1"|"not"|"notif0"|"notif1"|"pulldown"|"pullup"|"nmos"|"rnmos"|"pmos"|"rpmos"|"cmos"|"rcmos"|"tran"|"rtran"|"tranif0"|"rtranif0"|"tranif1"|"rtranif1"

%option noyywrap


  /* language parsing states */

%x Start
%x Commentt
%x FindBegin
%x StartComment
%%

.	{
    BEGIN(Start);  
	}


<Start>"//#"[^\n]+ {
                    QCString ss(verilogScanYYtext);
                    ss.stripPrefix("#");
					if(g_parseCode && !Config_getBool("STRIP_CODE_COMMENTS")){
					qlist.append(new QCString(ss.data()));
				    }
					ss.stripPrefix("//#");
                    c_lval.ctype=verilogScanYYtext[0];
                    addOneTextLine(ss);
                   }


<Start>{FEATURE} {
              //    fprintf(stderr,"<[ %s ]>",verilogScanYYtext);
				   if(!g_parseCode){
                   parseFeatures();
                 }
 
                 if(g_parseCode) {
                    strncpy(c_lval.cstr,verilogScanYYtext,verilogScanYYleng);
                    c_lval.cstr[verilogScanYYleng]='\0';                                                   
                    //writeFont("stringliteral",verilogScanYYtext);
                   qlist.append(new QCString(verilogScanYYtext));
                  }
                 }

<Start>{FEATURE1} {
				    QCString s(verilogScanYYtext);
				     VhdlDocGen::deleteAllChars(s,' ');
						      	
					if(g_parseCode){
                      QCString *yy=new QCString(verilogScanYYtext);
                      strncpy(c_lval.cstr,verilogScanYYtext,verilogScanYYleng);
                      c_lval.cstr[verilogScanYYleng]='\0';                                                   
                     qlist.append(yy);
                    }
                    
					if(!verilogDefineDict.find(s)){            
					if(!g_parseCode) addText(verilogScanYYtext,verilogScanYYleng);
					return  LETTER_TOKEN;
                   }
				   // REJECT;
					}

<Start>"ifnone"    {if(check())      return  IFNONE_TOKEN;REJECT;}
<Start>"realtime"    {if(check()) {yyPrevLine=yyLineNr;addText(verilogScanYYtext,verilogScanYYleng); return  REALTIME_TOKEN;}REJECT;}
<Start>"design"    {if(check())      return  DESIGN_TOKEN;REJECT;}

<Start>"library"{LF}+[^;]+    { 
                               QCString text(yytext);
							   if(g_parseCode){
							  printVerilogBuffer(true);
							    //writePrevVerilogWords();
								codifyLines(yytext);
							    }
								else
							   yyLineNr+=text.contains("\n");    
							   //addText(yytext,yyleng);
							   return  LIBRARY_TOKEN;
							   }

<Start>"config" {if(check())      return  CONFIG_TOKEN;REJECT;}
<Start>"endconfig" { if(check()){return  ENDCONFIG_TOKEN;} REJECT;}
<Start>"include"   {if(check())      return  INCLUDE_TOKEN;REJECT;}
<Start>"use"   {if(check())      return  USE_TOKEN;REJECT;}
<Start>"liblist"  { if(check())     return  LIBLIST_TOKEN;REJECT;}
<Start>"instance"   {if(check())      return  INSTANCE_TOKEN;REJECT;}
<Start>"cell"  { if(check())     return  CELL_TOKEN;REJECT;}
<Start>"showcancelled"   {if(check())      return  SHOWCANCEL_TOKEN;REJECT;}
<Start>"noshowcancelled"  { if(check())     return  NOSHOWCANCEL_TOKEN;REJECT;}
<Start>"pulsestyle_onevent"   {if(check())      return  PULSEONE_EVENT_TOKEN;REJECT;}
<Start>"pulsestyle_ondetect"  { if(check())     return  PULSEON_DETECT_TOKEN;REJECT;}
<Start>"edge"    {if(check())      return  EDGE_TOKEN;REJECT;}
<Start>"negedge" {if(check())      return  NEGEDGE_TOKEN;REJECT;}
<Start>"posedge" { if(check()){return  POSEDGE_TOKEN;} REJECT;}
<Start>"$fullskew"   {if(check())      return  FULLSKEW_TOKEN;REJECT;}
<Start>"$recrem"   {if(check())      return  RECREM_TOKEN;REJECT;}
<Start>"$removal"   {if(check())      return  REMOVAL_TOKEN;REJECT;}
<Start>"$timeskew"   {if(check())      return  TIMESKEW_TOKEN;REJECT;}
<Start>"$nochange"   {if(check())      return  NOCHANGE_TOKEN;REJECT;}
<Start>"$hold"   {if(check())      return  DHOLD_TOKEN;REJECT;}
<Start>"$setup"  { if(check())     return  DSETUP_TOKEN;REJECT;}
<Start>"$setuphold" {if(check())   return  DSETUPHOLD_TOKEN;REJECT;}
<Start>"$recovery" {if(check())    return  DRECOVERY_TOKEN;REJECT;}
<Start>"$skew"     {if(check())    return  DSKEW_TOKEN;REJECT;}
<Start>"$width"    {if(check())    return  DWIDTH_TOKEN;REJECT;}
<Start>"$period"   {if(check())    return  DPERIOD_TOKEN;REJECT;}
<Start>"time"      { if(check()){yyPrevLine=yyLineNr; addText(verilogScanYYtext,verilogScanYYleng);  return  TIME_TOKEN;} REJECT; }
<Start>"specparam" {if(check())    return  SPECPARAM_TOKEN;REJECT;}
<Start>"endspecify" {if(check())   return  ENDSPECIFY_TOKEN;REJECT;}
<Start>"specify"   {  if(check())   return  SPECIFY_TOKEN;REJECT;}
<Start>"end"        { if(check()) {   yyEndLine=yyLineNr;  strncpy(c_lval.cstr,verilogScanYYtext,verilogScanYYleng);c_lval.cstr[verilogScanYYleng]='\0';return  END_TOKEN;}REJECT;}

<Start>"fork"{LF}*":"{LF}*[a-zA-Z0-9_]+|"fork"       {
                            if(check(false)){    
                             strncpy(c_lval.cstr,verilogScanYYtext,verilogScanYYleng);
                             c_lval.cstr[verilogScanYYleng]='\0';
                             QCString qcs(verilogScanYYtext);
                               if(qcs.contains(':')>0){
                                 yyLineNr+=qcs.contains('\n');
                                   return  FORK_SPEC_TOKEN;
                                    }else
                                   return  FORK_TOKEN;
                                   }
                                  REJECT;
                                }


<Start>"begin"{LF}*":"{LF}*[a-zA-Z0-9_]+|"begin" {
 if(check(false)){    
 // qlist.append(new QCString(verilogScanYYtext));
  strncpy(c_lval.cstr,verilogScanYYtext,verilogScanYYleng);
  c_lval.cstr[verilogScanYYleng]='\0';
  QCString qcs(verilogScanYYtext);
  if(qcs.contains(':')>0){
  yyLineNr+=qcs.contains('\n');
  return  BEGIN_SPEC_TOKEN;
   }else
  return  BEGIN_TOKEN;
  }
 REJECT;
}

<Start>"generate" { if(check())     return  GENERATE_TOKEN;REJECT;}
<Start>"endgenerate" { if(check())     return  ENDGENERATE_TOKEN;REJECT;}
<Start>"genvar" { if(check())     return  GENVAR_TOKEN;REJECT;}
<Start>"default" { if(check())     return  DEFAULT_TOKEN;REJECT;}
<Start>"automatic" { if(check()){ addText(verilogScanYYtext,verilogScanYYleng);     return  AUTO_TOKEN;}REJECT;}
<Start>"signed" { if(check()){ yyPrevLine=yyLineNr; addText(verilogScanYYtext,verilogScanYYleng);    return  SIGNED_TOKEN;}REJECT;}
<Start>"default" { if(check())     return  DEFAULT_TOKEN;REJECT;}
<Start>"endcase" { if(check())     return  ENDCASE_TOKEN;REJECT;}
<Start>"else"     { if(check())    return  ELSE_TOKEN; REJECT;}
<Start>"while"    { if(check())    return  WHILE_TOKEN; REJECT;}
<Start>"repeat"   { if(check())    return  REPEAT_TOKEN;REJECT;}
<Start>"forever"  { if(check())    return  FOREVER_TOKEN;REJECT;}
<Start>"casez"    { if(check())    return  CASEZ_TOKEN; REJECT;}
<Start>"casex"    { if(check())    return  CASEX_TOKEN;REJECT;}
<Start>"case"     { if(check())    return  CASE_TOKEN;REJECT;}
<Start>"if"       { if(check())    return  IF_TOKEN;REJECT;}
<Start>"disable"  { if(check())    return  DISABLE_TOKEN;REJECT;}
<Start>"deassign" { if(check())    return  DEASSIGN_TOKEN;REJECT;}
<Start>"release"  { if(check())    return  RELEASE_TOKEN; REJECT;}
<Start>"force"    { if(check())    return  FORCE_TOKEN; REJECT; }
<Start>"wait"     { if(check())    return  WAIT_TOKEN; REJECT;}
<Start>"join"     { if(check())    return  JOIN_TOKEN; REJECT;}
<Start>"for"      { if(check())    return  FOR_TOKEN; REJECT;}
<Start>"always"   { if(check()) {  yyPrevLine=yyLineNr;  return  ALWAYS_TOKEN;}REJECT;}
<Start>"endfunction"  { if(check()){yyPrevLine=yyLineNr;  return  ENDFUNC_TOKEN;} REJECT;}
<Start>"function"     { if(check()){yyPrevLine=yyLineNr;  return  FUNC_TOKEN;} REJECT;}
<Start>"endtask"      { if(check()){yyPrevLine=yyLineNr;  return  ENDTASK_TOKEN;}REJECT;}
<Start>"task"         { if(check()){yyPrevLine=yyLineNr;  return  TASK_TOKEN;} REJECT;}
<Start>"table"        { if(check())  return  TABLE_TOKEN; REJECT;}
<Start>"endtable"     { if(check())  return  ENDTABLE_TOKEN; REJECT;}
<Start>"initial"      { if(check()) {return  INITIAL_TOKEN;} REJECT;}
<Start>"endprimitive" { if(check())  return  ENDPRIMITIVE_TOKEN;REJECT;}
<Start>"primitive"    { if(check()){yyPrevLine=yyLineNr;  return  PRIMITIVE_TOKEN;} REJECT;}
<Start>"macromodule"  { if(check())  return  MACRO_MODUL_TOKEN; REJECT;}
<Start>"module"       { if(check())   { yyPrevLine=yyLineNr; return  MODUL_TOKEN; } REJECT;}
<Start>"endmodule"   { if(check())    return  ENDMODUL_TOKEN;REJECT;}
<Start>"reg"         { if(check()) {yyPrevLine=yyLineNr; addText(verilogScanYYtext,verilogScanYYleng);return  REG_TOKEN; } REJECT;}
<Start>"integer"     { if(check()) {yyPrevLine=yyLineNr; addText(verilogScanYYtext,verilogScanYYleng);return  INTEGER_TOKEN;}REJECT;}
<Start>"defparam" { if(check()) return  DEFPARAM_TOKEN; REJECT;}
<Start>"real"     { if(check()) {yyPrevLine=yyLineNr; addText(verilogScanYYtext,verilogScanYYleng);   return  REAL_TOKEN; } REJECT; }
<Start>"event"    { if(check()) return  EVENT_TOKEN;REJECT;}
<Start>"assign"   { if(check()) return  ASSIGN_TOKEN; REJECT;}
<Start>"scalared" { if(check()) return  SCALAR_TOKEN;REJECT;}
<Start>"vectored" { if(check()) return  VEC_TOKEN;   REJECT;}
<Start>"small"    { if(check()) return  SMALL_TOKEN;REJECT;}
<Start>"medium"   { if(check()) return  MEDIUM_TOKEN; REJECT;}
<Start>"large"    { if(check()) return  LARGE_TOKEN; REJECT;}
<Start>"output"   { if(check())  {yyPrevLine=yyLineNr;  addText(verilogScanYYtext,verilogScanYYleng);   return  OUTPUT_TOKEN; }REJECT;}
<Start>"input"    { if(check()) {yyPrevLine=yyLineNr; addText(verilogScanYYtext,verilogScanYYleng); return  INPUT_TOKEN;} REJECT;}
<Start>"inout"    { if(check()) {yyPrevLine=yyLineNr; addText(verilogScanYYtext,verilogScanYYleng);    return  INOUT_TOKEN; }REJECT;}
<Start>"parameter" { if(check()) {yyPrevLine=yyLineNr;return  PARAMETER_TOKEN;}REJECT;}
<Start>"localparam" { if(check()) {yyPrevLine=yyLineNr; return  LOCALPARAM_TOKEN;}REJECT;}
<Start>{NETTYPE}   { if(check()) { yyPrevLine=yyLineNr; addText(verilogScanYYtext,verilogScanYYleng);return  NET_TOKEN;} REJECT;}
<Start>{STRENGTH0} { if(check()) {  addText(verilogScanYYtext,verilogScanYYleng);   return  STR0_TOKEN;} REJECT;}
<Start>{STRENGTH1} { if(check()) {  addText(verilogScanYYtext,verilogScanYYleng); return  STR1_TOKEN;}REJECT;}
<Start>{GATETYPE}  { if(check()) {  addText(verilogScanYYtext,verilogScanYYleng);  return  GATE_TOKEN;}REJECT;}
<Start>{COMMENT}|{COMMENT1}   { 
                      QCString text(verilogScanYYtext);
					 if(!g_parseCode) {       
					  if(text.stripPrefix("//%"))             
					  VerilogDocGen::setBrief(text,yyLineNr);
					  }
					 
					 if(g_parseCode)
					 { 
					   int b=text.contains("//%");
					   if(Config_getBool("STRIP_CODE_COMMENTS") && !b) 
					     qlist.append(new QCString(verilogScanYYtext));
					   else	if(!Config_getBool("STRIP_CODE_COMMENTS"))  
					      qlist.append(new QCString(verilogScanYYtext));
					  } 
					 c_lval.ctype=verilogScanYYtext[0];
					 }
<Start>{STRING}    {
                  //   fprintf(stderr,"\n [%s : %c %d] string\n",verilogScanYYtext,verilogScanYYtext[verilogScanYYleng-2],verilogScanYYleng);

                      if((verilogScanYYleng > 2) && verilogScanYYtext[verilogScanYYleng-2] != 92) {
                      addText(verilogScanYYtext,verilogScanYYleng);
                      if(g_parseCode) { qlist.append(new QCString(verilogScanYYtext));c_lval.ctype=verilogScanYYtext[0];}
                      return  STRING_TOKEN;
                      }
                      if(verilogScanYYleng == 2){ 
                         if(g_parseCode) { qlist.append(new QCString(verilogScanYYtext)); }   
                        return  STRING_TOKEN;
                        } 
                     unput(verilogScanYYtext[verilogScanYYleng-1]);
                     yymore();
                     }

<Start>"\t"|"\r" {
                   if(yytext[0]=='\t'){
                    if(!g_parseCode)
                    parseToken(" ");
                   else
                    parseToken(verilogScanYYtext);
                  }
                  c_lval.ctype=' ';} 

<Start>[\n] {
   	   		// fprintf(stderr,"\nparse line of code : [line: %d]",yyLineNr);
               addToken('\n');
              
			   if(g_parseCode)qlist.append(new QCString(verilogScanYYtext)); 
                else
	           yyLineNr+=QCString(verilogScanYYtext).contains('\n');
	           c_lval.ctype=verilogScanYYtext[0];
               }

<Start>[ ]+  { parseToken(verilogScanYYtext);}
<Start>[0-9]+|[0-9][0-9_]+|[0-9]+{BASE}[0-9a-zA-Z_?]+|[0-9]+"."[0-9eE]+|{BASE}[0-9a-zA-Z_?]+ {
                                                      addText(verilogScanYYtext,verilogScanYYleng);
                                                      strncpy(c_lval.cstr,verilogScanYYtext,verilogScanYYleng);
                                                      c_lval.cstr[verilogScanYYleng]='\0';
                                                      if(g_parseCode) qlist.append(new QCString(verilogScanYYtext)); 
					                                  return  DIGIT_TOKEN;
                                                     }

<Start>[a-zA-Z]+|[_a-zA-Z$][_a-zA-Z0-9$]+ {
						                 addText(verilogScanYYtext,verilogScanYYleng);
						                 char c=yy_hold_char;
						   	           // yyPrevLine=yyLineNr;
										 if(g_parseCode){ 
										   int ll=qlist.count();
										   qlist.append(new QCString(verilogScanYYtext));
						                  }
						                   strncpy(c_lval.cstr,verilogScanYYtext,verilogScanYYleng);
                                           c_lval.cstr[verilogScanYYleng]='\0'; 
						                  
						                
						                  c_lloc.first_line=yyLineNr;
						                  return  LETTER_TOKEN;
                                         }

 
<Start>.  {
  
  c_lval.ctype=verilogScanYYtext[0];
  yyless(0);
  REJECT;
 // return  CHAR_TOKEN;
 }         

<Start>"!"   {  parseToken(verilogScanYYtext);  return   EXCLAMATION_TOKEN;}
<Start>"_"   {  parseToken(verilogScanYYtext);   return   UNDERSCORE_TOKEN;}
<Start>";"   {  parseToken(verilogScanYYtext); return  SEM_TOKEN;}
<Start>"."   {  parseToken(verilogScanYYtext);  return  DOT_TOKEN;}
<Start>","   {  parseToken(verilogScanYYtext); return  COMMA_TOKEN;}
<Start>"?"   {  parseToken(verilogScanYYtext);return  QUESTION_TOKEN;}
<Start>"+"   {  parseToken(verilogScanYYtext);return  PLUS_TOKEN;}
<Start>"-"   {  parseToken(verilogScanYYtext);return  MINUS_TOKEN;}
<Start>":"   {   parseToken(verilogScanYYtext); return  COLON_TOKEN;}
<Start>"("   { parseToken(verilogScanYYtext);if(g_parseCode) { c_lval.cstr[0]=' ';} return  RBRACE_TOKEN;}
<Start>")"   { parseToken(verilogScanYYtext);return  LBRACE_TOKEN;}
<Start>"}"   { parseToken(verilogScanYYtext);return  LRAM_TOKEN;}
<Start>"{"   { parseToken(verilogScanYYtext);return  RRAM_TOKEN;}
<Start>"["   {  parseToken(verilogScanYYtext);return  RBRACKET_TOKEN;}
<Start>"]"   { parseToken(verilogScanYYtext);return  LBRACKET_TOKEN;}
<Start>"&"   { parseToken(verilogScanYYtext);return  AND_TOKEN;}
<Start>"|"   { parseToken(verilogScanYYtext); return  OR_TOKEN;}
<Start>"="   { parseToken(verilogScanYYtext);  return  EQU_TOKEN;}
<Start>"<"   { parseToken(verilogScanYYtext); return  GT_TOKEN;}
<Start>">"   { parseToken(verilogScanYYtext);return  LT_TOKEN;}
<Start>"^"   { parseToken(verilogScanYYtext);return  NOT_TOKEN;}
<Start>"~"   { parseToken(verilogScanYYtext); return  SNAKE_TOKEN;}
<Start>"*"   { parseToken(verilogScanYYtext);return  MULT_TOKEN;}
<Start>"%"   { parseToken(verilogScanYYtext);return  PERCENTAL_TOKEN;}
<Start>"/" {
 						 char c=yy_hold_char;
                         if(c !='/'){
						 parseToken(verilogScanYYtext);
						 return  ENV_TOKEN;
                         }
						 unput('/');
						 REJECT;}

<Start>"@" { parseToken(verilogScanYYtext);return  AT_TOKEN;}
<Start>"#" { parseToken(verilogScanYYtext);	return  PARA_TOKEN;}
<Start>"$" { if(g_parseCode) parseToken(verilogScanYYtext); return  DOLLAR_TOKEN;}
<Start>"/*" {
                 verilogBufferClear(); 
				 addText(verilogScanYYtext,verilogScanYYleng);
				 startComment=yyLineNr;
				 BEGIN(StartComment); 
				 }

<StartComment>[^*]*[*]+    {
                       QCString tt(verilogScanYYtext);
					   addText(verilogScanYYtext,verilogScanYYleng);
                       char c=yy_hold_char;
                         if(c =='/'){
						   unput('*');
						   BEGIN(Commentt);
						 }
					     else BEGIN(StartComment);
					 }

<Commentt>"*/" {	 
						 int proto=0;
							 bool needEntry=false;
                              if(!g_parseCode){ 
							      
							 QCString qcs(getVerilogString());
							 yyLineNr+=qcs.contains('\n');
							 Entry *pp = new Entry;			 
							 pp->name="string";
						     pp->startLine=yyLineNr;
							 pp->bodyLine=yyLineNr;
							 
							 bool gstat=parseCommentBlock(
								g_thisParser,    // parser
								pp,              // Entry
								qcs,             // text
								yyFileName,      // file
								startComment,     // startLine of comment
							    FALSE,
								FALSE,
								FALSE,
								curRoots->protection,  // public
								proto,
								needEntry );
							
							if(pp->section & Entry::VARIABLE_SEC ) {pp->section=Entry::OVERLOADDOC_SEC;}
							if(pp->section & Entry::COMPOUNDDOC_MASK ) 
                             {  
								  pp->section=(Entry::STRUCTDOC_SEC);// | Entry::OVERLOADDOC_SEC) ;
						    	  pp->bodyLine=-1;
							 }
							
							if(qcs.isEmpty() && pp)
							{ 
							  delete pp;
							  pp=0;
							 }
							else {  
							 pp->doc=stripLeadingAndTrailingEmptyLines(pp->doc);	
							 if(!needEntry )
							   pp->name="string";
							  Entry* ent=getCurrVerilogEntry();
							   if(ent)
							    ent->addSubEntry(pp);
							  else
							   curRoots->addSubEntry(pp);
							 }
							}//if
							
                            if(g_parseCode){ 
						     QCString *qq=new QCString(getVerilogString());
							qlist.append(qq);		
							}
                      	 verilogBufferClear();
                         BEGIN(Start);
             }
%%

//------ ------------------------------------------------------------------------------------------------

 int MyParserConv::doLex(){
   int token;
   token=yylex();
   return token;
  } 
 
 void resetScanner(const char* s,MyParserConv* parse) { }


 void verilogBufferClear() 
 {
  int i;
  i=getVerilogToken();
  memset(&g_buf[0],'\0',iSize);
  iSize=0;
  if(i==LETTER_TOKEN){
   lastLetter=verilogScanYYtext;
 
   }
  }

 void addToken (char c)
 {
    if(iSize>inputPosition)
	  {
	  verilogBufferClear();
	 // assert(0);
	 }
      g_buf[iSize]=c;
      iSize++;
      g_buf[iSize]='\0'; 

 } 

 QCString getLastLetter(){ return lastLetter; }

 void  addText (char *word, int llen)
 {
  	  while(llen>0)
      {
      g_buf[iSize]=*word;
      *word++;
      iSize++;
      llen--;
      }    
     g_buf[iSize]='\0'; 
  } 

const char* getVerilogString() {if(iSize==0) return NULL;return &g_buf[0];}
char* getVerilogParsingFile(){return yyFileName.data();}
QList<Entry>& getVerilogFeatureList(){ return qfeatlist;}

int getVerilogLine() { return yyLineNr; }
int getVerilogPrevLine() { return yyPrevLine; }
int getVerilogEndLine(){ return yyEndLine; };


void VerilogScanner::resetCodeParserState(){}
bool VerilogScanner::needsPreprocessing(const QCString &extension){ return false; }
void VerilogScanner::parsePrototype(const char *text){ }

void VerilogScanner::parseInput(const char *fileName,const char *fileBuf,Entry *root)
{
     QCString pPuffer(fileBuf);
	 pPuffer.prepend(" ");
	 if(!g_lexInit)
	 buildKeyMap();
	 inputFile.setName(fileName);
	 yyFileName=QCString(fileName);
	 VerilogDocGen::initVerilogParser();
	 if (!inputFile.open(IO_ReadOnly))
	  {
       fprintf(stderr,"\n\n could not open file: %s !!\n\n",yyFileName.data());
       return ;
	  }
       if(g_lexInit)
		 verilogScanYYrestart( verilogScanYYin );
	   g_lexInit=TRUE;
	   curRoots=root;	
 	   setGlobalVerilogEntry(curRoots,false);
       g_thisParser=this;
	   iSize=0;
       int len =inputFile.size();
       inputPosition=0;
	   g_buf=new char[len+1024];
       assert(g_buf);
       inString=pPuffer.data();
       g_inputLines  = countLines();
	   totLines=g_inputLines;
	   yyLineNr=1;
       pconv=new MyParserConv();
       resetScanner(NULL,pconv);
       assert(pconv);
       int ok=pconv->parse(pconv);
	    VerilogDocGen::addIncludes();
	   qfeatlist.clear();
	   delete pconv;
	   delete [] g_buf;g_buf=0;
	  
	  	  
}

 void VerilogScanner::parseCode(CodeOutputInterface &codeOutIntf,
                   const char *scopeName,
                   const QCString &input,
				   bool isExampleBlock,
                   const char *exampleName,
                   FileDef *fileDef,
                   int startLine,
                   int endLine,
                   bool inlineFragment,
                   MemberDef *memberDef
                  )
				  { 
				     QCString pPuffer(input.data());
                	 pPuffer.prepend(" ");
              
                   VerilogDocGen::initVerilogParser();
				   if(memberDef)
                   {
                    ClassDef *dd=memberDef->getClassDef();
	                if(dd)
	                g_CurrClass=dd->className();
                    VerilogDocGen::setCurrVerilogClass(g_CurrClass);
						startLine--;
            
                   }
                      g_code = &codeOutIntf;
                      inString   = pPuffer.data();
                      inputPosition = 0;
					  iSize=0;
                      g_buf=new char[input.length()+1024];
                      assert(g_buf);
					 
                      setGlobalVerilogEntry(0,true);
     
					  g_currentFontClass = 0;
                      g_needsTermination = FALSE;
 
                      if (endLine!=-1)
                      g_inputLines  = endLine+1;
					    else
                      g_inputLines  = countLines();
                      totLines=g_inputLines;
				      if (startLine!=-1)
                        yyLineNr    = startLine;
                      else
                        yyLineNr    = 1;
                         g_exampleName   = exampleName;
                        g_sourceFileDef = fileDef;
                        yyFileName=fileDef->fileName();
                        if (isExampleBlock && fileDef==0)
                        {
                          // create a dummy filedef for the example
                         g_sourceFileDef = new FileDef("",exampleName);
                        }
 
                       if (g_sourceFileDef) 
                       {
                        setCurrentDoc(g_sourceFileDef->name(),g_sourceFileDef->getSourceFileBase());
                       }
 
                       g_currentDefinition = 0;
                       g_currentMemberDef = 0;
               
  
                      if (!g_exampleName.isEmpty())
                      {
                       g_exampleFile = convertNameToFile(g_exampleName+"-example");
                       }
  
                      g_includeCodeFragment = inlineFragment;
                        if(!memberDef) startCodeLine();
             
                     
  		             verilogScanYYrestart( verilogScanYYin );
				    g_parseCode=true;
                    MyParserConv conv;
 	                resetScanner(input.data(),&conv);
                     int ok=conv.parse(&conv);
					if (isExampleBlock && g_sourceFileDef)
                    {
                     // delete the temporary file definition used for this example
                       delete g_sourceFileDef;
                       g_sourceFileDef=0;
                    }
                   
				    printVerilogBuffer(true);
				    g_parseCode=false;
				    delete [] g_buf;g_buf=0;
				   return;
                    }

//-------------------------------------------------------------------------------------------------------

static void parseFeatures(){
        QCString tmp,args,name;
		QCString yy(verilogScanYYtext);
        int specLine=yyLineNr;
	   //printf("\n >FEATURE<%s",verilogScanYYtext);
       if(yy.stripPrefix("`define")){
      	  int index=yy.find("//#"); // any docs in this line?
		  if(index>1){
			tmp=yy.mid(index,yy.length());
			tmp.stripPrefix("//#");
			yy=yy.left(index-1);
		  }
		  else
		  {
			 int index=yy.find("//");
			 if(index>1)
			   yy=yy.left(index-1);
			 Entry* pEnt=VerilogDocGen::getCurrBrief();
			 if(!pEnt->brief.isEmpty()){
               specLine=pEnt->briefLine;
			   tmp=pEnt->brief;
			   pEnt->reset();
			 }
		  }
         
		while(yy.stripPrefix(" "));
	    QRegExp reg("[^a-zA-Z_0-9$]+");
	    int ll=yy.find(reg,0);
		 if(ll>0){
			 args=yy.mid(ll,yy.length());
		     name=yy.left(ll);
			 name.prepend("`");
			 }
		  
       Entry* pTemp=VerilogDocGen::makeNewEntry(name.data(),Entry::VARIABLE_SEC,VerilogDocGen::FEATURE,yyLineNr);
       pTemp->type="";
       pTemp->args=args;
       if(!tmp.isEmpty()){
        pTemp->brief=tmp;
		pTemp->briefLine=specLine; //yyLineNr;
	   }
         qfeatlist.append(pTemp);
      
  }
  if(yy.stripPrefix("`include")){
   VhdlDocGen::deleteAllChars(yy,' ');
   VhdlDocGen::deleteAllChars(yy,'"');
    if(yy.isEmpty())return;
	QStringList ql=QStringList::split('/',yy,false);
    yy=(QCString)ql.last();
    int j=yy.find(".");
    if(j>0)
     yy=yy.left(j);
   Entry* pTemp=VerilogDocGen::makeNewEntry(yy.data(),Entry::VARIABLE_SEC,VerilogDocGen::INCLUDE,yyLineNr);
  
   Entry* pEnt=VerilogDocGen::getCurrBrief();
			 if(!pEnt->brief.isEmpty()){
               pTemp->briefLine=pEnt->briefLine;
			   pTemp->brief=pEnt->brief;
			   pEnt->reset();
			 }
  
  pTemp->type="include";
  qfeatlist.append(pTemp);
  }
}// parseFeatures

void mapVerilogFeatures(Entry* cur){
 uint len=qfeatlist.count();
 for(uint j=0;j<len;j++){
  Entry* p=(Entry*)qfeatlist.at(j);
  if(p){
   Entry *ee=new Entry(*p);
   cur->addSubEntry(ee);
  }
 }//for
  for(uint i=0;i<len;i++){
  Entry* p=(Entry*)qfeatlist.at(i);
  {
   // delete only `define, treat `include global
   if(p->spec==VerilogDocGen::FEATURE) 
    { 
     qfeatlist.remove(i);
     len--;
     i=0;
     }
  }
 }//for 
}//mapFeature

