#ifndef verilogscanner_h
#define verilogscanner_h

#include "parserintf.h"
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include <ctype.h>

#include <qarray.h>
#include <qstack.h>
#include <unistd.h>
#include <qfile.h>
#include <qdict.h>

#include "entry.h"
#include "qcstring.h"
#include "qlist.h"
#include "qstringlist.h"

#include "filedef.h"
#include "classlist.h"
#include "classdef.h"

#include "translator.h"

#include "qregexp.h"
#include "outputlist.h"
#include "membername.h"
#include "memberdef.h"
#include "memberlist.h"
#include "VerilogDocGen.h"
#include "VerilogParser.hpp"


class VerilogScanner : public ParserInterface
{
  public:
    virtual ~VerilogScanner() {}
    void parseInput(const char * fileName, 
                    const char *fileBuf, 
                    Entry *root);
    bool needsPreprocessing(const QCString &extension);
    void parseCode(CodeOutputInterface &codeOutIntf,
                   const char *scopeName,
                   const QCString &input,
                   bool isExampleBlock,
                   const char *exampleName=0,
                   FileDef *fileDef=0,
                   int startLine=-1,
                   int endLine=-1,
                   bool inlineFragment=FALSE,
                   MemberDef *memberDef=0
                  );
    void resetCodeParserState();
    void parsePrototype(const char *text);
};

void codifyVerilogString(char* c,char* color=NULL);
bool generateVerilogMemLink(QCString &clName,QCString& memberName,int type);
bool generateVerilogClassOrGlobalLink(char *clName);
void  verilogBufferClear();
const char* getVerilogString();
int getVerilogLine();
int getVerilogPrevLine();
int getVerilogEndLine();
int getVerilogToken(); 
char* getVerilogParsingFile();


void printVerilogBuffer(bool b=false);
void printVerilogStringList();
void writePrevVerilogWords(QCString& qcs);

 
Entry* getCurrVerilogEntry();

 //------ function for parsing code ---------------------------------
 
void writeVerilogFont(const char *s,const char* text);
void generateVerilogMemLink(QCString &clName,QCString& memberName);
bool writeVerilogColoredWord(QCString& word );
void mapVerilogFeatures(Entry* cur);
void setGlobalVerilogEntry(Entry* ee,bool pc);


QCString getLastLetter();
QList<Entry>& getVerilogFeatureList();
#endif